__version__ = "0.5.7.1"

__all__ = ["cursorUp", "cursorDown", "cursorRight", "cursorLeft", "scrollUp", "scrollDown", "setWindowTitle", "overwriteLines", "createLoadBar", "truncate", "goto", "styles"]

from digiformatter.formatter import cursorUp, cursorDown, cursorRight, cursorLeft, scrollUp, scrollDown, setWindowTitle, overwriteLines, createLoadBar, truncate, goto
from digiformatter.styles import styles
